@@ -1,6 +1,6 @@
-package MooX::HandlesVia;
+  package MooX::HandlesVia;
 {
-  $MooX::HandlesVia::VERSION = '0.001000';
+  $MooX::HandlesVia::VERSION = '0.001001';
 }
 # ABSTRACT: NativeTrait-like behavior for Moo.
 
@@ -16,6 +16,7 @@ my %RESERVED = (
   'Number' => 'Data::Perl::Number::MooseLike',
   'Code' => 'Data::Perl::Code',
 );
+my %REVERSED = reverse %RESERVED;
 
 sub import {
   my ($class) = @_;
@@ -36,12 +37,14 @@ sub process_has {
   my $handles = $opts{handles};
   return ($name, %opts) if not $handles or ref $handles ne 'HASH';
 
-  if (my $via = $opts{handles_via}) {
+  if (my $via = delete $opts{handles_via}) {
     # try to load the reserved mapping, if it exists, else the full name
     $via = $RESERVED{$via} || $via;
-
     require_module($via);
 
+    # clone handles for HandlesMoose support
+    my %handles_clone = %$handles;
+
     while (my ($target, $delegation) = each %$handles) {
       # if passed an array, handle the curry
       if (ref $delegation eq 'ARRAY') {
@@ -56,6 +59,19 @@ sub process_has {
         }
       }
     }
+
+    # install our support for moose upgrading of class/role
+    # we deleted the handles_via key above, but install it as a native trait
+    my $inflator = $opts{moosify};
+    $opts{moosify} = sub {
+      my ($spec) = @_;
+
+      $spec->{handles} = \%handles_clone;
+      $spec->{traits} = [$REVERSED{$via} || $via];
+
+      # pass through if needed
+      $inflator->($spec) if ref($spec) eq 'CODE';
+    };
   }
 
   ($name, %opts);
@@ -63,6 +79,8 @@ sub process_has {
 
 1;
 
+
+
 =pod
 
 =head1 NAME
@@ -71,7 +89,7 @@ MooX::HandlesVia - NativeTrait-like behavior for Moo.
 
 =head1 VERSION
 
-version 0.001000
+version 0.001001
 
 =head1 SYNOPSIS
 
@@ -194,6 +212,7 @@ the same terms as the Perl 5 programming language system itself.
 
 =cut
 
+
 __END__
 ==pod
 
